package com.htfp.weather.web.config.interceptor;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Formatter;

/**
 * @Author : shiyi
 * @Date : 2024/7/5 10:26
 * @Description : 请求过滤
 */
@Slf4j @Component
public class InterfaceInterceptor implements HandlerInterceptor {

    private static final Integer SECONDS = 60 * 1000;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String servletPath = request.getServletPath();
        // log.info("InterfaceInterceptor.preHandle >> 进入拦截, {}, {}", servletPath, handler.getClass().getName());
        // ip 校验
        if (checkIp(request)) {
            return true;
        }
        // 签名校验
        final String signature = request.getHeader("signature");
        final String timestamp = String.valueOf(request.getHeader("timestamp"));
        final String requestURI = request.getRequestURI();
        String checkSignature = createSignature(timestamp, requestURI);

        if (!checkSignature.equalsIgnoreCase(signature)) {
            return false;
        }

        //判断时间是否是60s内
        long nowTime = System.currentTimeMillis();
        long reqTime = Long.parseLong(timestamp);
        if (Math.abs(nowTime - reqTime) > SECONDS) {
            return false;
        }

        return true;
    }

    private boolean checkIp(HttpServletRequest request) {
        String remoteAddr = request.getRemoteAddr();
        if ("127.0.0.1".equals(remoteAddr) || "0:0:0:0:0:0:0:1".equals(remoteAddr)) {
            return true;
        }
        if (remoteAddr.startsWith("172.10.0.")) {
            // 局域网测试
            return true;
        }
        if ("182.92.130.23".equals(remoteAddr)) {
            // 本地服务
            return true;
        }
        return false;
    }

    /**
     * @param key - 时间戳（需传入字符串）
     * @param data- 接口路径，例如 /htfp/weather/upper/queryPlaneGrid
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     */
    private String createSignature(String key, String data) throws NoSuchAlgorithmException, InvalidKeyException {
        // TODO 签名算法待定
        SecretKeySpec signingKey = new SecretKeySpec(key.getBytes(),  "HmacSHA1");
        Mac mac = Mac.getInstance("HmacSHA1");
        mac.init(signingKey);
        byte[] bytes = mac.doFinal(data.getBytes());
        return toHexString(bytes);
    }

    /**
     * 16进制编码，便于地面站发送请求
     * @param bytes
     * @return
     */
    private  String toHexString(byte[] bytes) {
        Formatter formatter = new Formatter();
        for (byte b : bytes) {
            formatter.format("%02x", b);
        }
        return formatter.toString();
    }

}

