package com.htfp.weather.web.controller;

import com.htfp.weather.web.config.aspect.ControllerLog;
import com.htfp.weather.web.pojo.hefeng.HeFengWarningResponse;
import com.htfp.weather.web.pojo.request.Position2D;
import com.htfp.weather.web.pojo.response.NowWeatherStatus;
import com.htfp.weather.web.pojo.response.Result;
import com.htfp.weather.web.pojo.response.SurfaceWeatherWarning;
import com.htfp.weather.web.pojo.response.TimeSeriesDataset;
import com.htfp.weather.web.service.surfaceapi.CaiYunServiceImpl;
import com.htfp.weather.web.service.surfaceapi.CmaServiceImpl;
import com.htfp.weather.web.service.surfaceapi.HeFengServiceImpl;
import com.htfp.weather.web.service.IDataService;
import com.htfp.weather.web.service.surfaceapi.ISurfaceDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author : shiyi
 * @Date : 2024/1/24 16:07
 * @Description : 地面天气状态
 */
@CrossOrigin
@RestController
@Slf4j
@RequestMapping("/htfp/weather/surface/")
public class SurfaceWeatherController {
    @Resource(name = "hefeng")
    ISurfaceDataService surfaceDataService;

    @Resource
    CmaServiceImpl cmaService;

    @PostMapping("/querySurfaceNowWeather")
    @ControllerLog(info = "地面实时气象信息查询")
    public Result queryNowWeather(@Validated @RequestBody Position2D position2D) throws Exception {
        double lat = position2D.getLatitude();
        double lon = position2D.getLongitude();

        NowWeatherStatus nowWeatherStatus = surfaceDataService.getNowSurfaceWeatherStatus(lat, lon);
        nowWeatherStatus.setLatitude(lat);
        nowWeatherStatus.setLongitude(lon);
        return Result.success(nowWeatherStatus);
    }

    @PostMapping("/querySurfaceForecast")
    @ControllerLog(info = "地面24小时预报结果查询")
    public Result querySurfaceForecast(@Validated @RequestBody Position2D position2D) throws Exception {
        double lat = position2D.getLatitude();
        double lon = position2D.getLongitude();

        TimeSeriesDataset forecastSeries = surfaceDataService.getForecastSeries(lat, lon);
        forecastSeries.setLatitude(lat);
        forecastSeries.setLongitude(lon);
        return Result.success(forecastSeries);
    }

    @PostMapping("/queryWeatherWarning")
    @ControllerLog(info = "地面气象预警信息查询")
    public Result queryWeatherWarning(@Validated @RequestBody Position2D position2D) throws Exception {
        double lat = position2D.getLatitude();
        double lon = position2D.getLongitude();

        // List<SurfaceWeatherWarning> warning = surfaceDataService.getSurfaceWarning(lat, lon);
        List<SurfaceWeatherWarning> warning = cmaService.getSurfaceWarning(lat, lon);
        return Result.success(warning);
    }

}
